"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.networkRefreshTokenHandler = void 0;
const axios_1 = __importDefault(require("axios"));
const ax = axios_1.default.create({
    baseURL: 'http://localhost:8080',
    timeout: 20000,
});
/**
 * Network refresh token storage handler.
 * Compliant with <strong><em>/api/setdata</em></strong> and <strong><em>/api/getdata</em></strong> endpoints of Kiosk Local Service.
 */
exports.networkRefreshTokenHandler = {
    setRefreshToken(tenant, refreshToken) {
        return ax
            .post('/api/setdata', {
            createdAt: Date.now(),
            updatedAt: Date.now(),
            name: `${tenant}_refresh_token`,
            value: refreshToken,
        })
            .then(() => {
            return;
        })
            .catch((err) => {
            console.error('error saving refresh token', err);
            return;
        });
    },
    getRefreshToken(tenant) {
        return ax
            .post('/api/getdata', { name: `${tenant}_refresh_token` })
            .then(({ data }) => data.value)
            .catch((err) => {
            console.error('error getting refresh token', err);
            return '';
        });
    },
    clearRefreshToken(tenant) {
        return ax
            .post('/api/setdata', {
            createdAt: Date.now(),
            updatedAt: Date.now(),
            name: `${tenant}_refresh_token`,
            value: '',
        })
            .then(() => {
            return;
        })
            .catch((err) => {
            console.error('error removing refresh token', err);
            return;
        });
    },
};
