var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import React, { Fragment, useContext, useEffect, useMemo, useReducer, useState, } from 'react';
import AuthContext from './AuthContext';
import ConfigContext from './ConfigContext';
import * as orderingCore from '@orderingstack/ordering-core';
import axios from 'axios';
import { getErrorMessage } from './utils';
import { QRCodeSVG } from 'qrcode.react';
import { FadeLoader } from 'react-spinners';
import { jwtDecode } from 'jwt-decode';
function Loader() {
    return React.createElement(FadeLoader, null);
}
function QRLoader({ size }) {
    const height = Math.min(size / 10, 25);
    const width = Math.min(size / 30, 5);
    const radius = Math.min(height, width) / 2;
    return (React.createElement("div", { style: {
            width: size,
            height: size,
            display: 'flex',
            alignItems: 'center',
            justifyContent: 'center',
        } },
        React.createElement(FadeLoader, { height: height, width: width, radius: radius })));
}
const _refreshStorageHandler = {
    getRefreshToken: (tenant) => {
        return (window.localStorage.getItem(`orderingstack_${tenant}_refreshToken`) || '');
    },
    setRefreshToken: (tenant, token) => {
        window.localStorage.setItem(`orderingstack_${tenant}_refreshToken`, token);
    },
    clearRefreshToken: (tenant) => {
        window.localStorage.removeItem(`orderingstack_${tenant}_refreshToken`);
    },
};
function getModuleConfig(baseUrl, accessToken) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const tokenData = jwtDecode(accessToken);
            if (tokenData.MODULE) {
                const { data } = yield axios.get(`${baseUrl}/auth-api/api/module-config`, {
                    headers: {
                        Authorization: `Bearer ${accessToken}`,
                    },
                });
                return data;
            }
        }
        catch (e) {
            console.warn('JWT decode failed', e);
            return undefined;
        }
    });
}
function DefaultDeviceCodeComp(props) {
    const { error, loading, data } = props;
    const qrCode = data === null || data === void 0 ? void 0 : data.verification_uri_complete;
    const code = data === null || data === void 0 ? void 0 : data.user_code;
    const size = Math.round(Math.min(window.innerHeight, window.innerWidth) / 3);
    const [seconds, setSeconds] = useState(0);
    useEffect(() => {
        if (data === null || data === void 0 ? void 0 : data.exp) {
            const timer = setInterval(() => setSeconds(Math.max(Math.round((data.exp - Date.now()) / 1000), 0)), 1000);
            return () => clearInterval(timer);
        }
        else {
            setSeconds(0);
        }
    }, [data]);
    return (React.createElement("div", { style: {
            width: '100%',
            height: '100vh',
            display: 'flex',
            flexDirection: 'column',
            alignItems: 'center',
            justifyContent: 'center',
            fontSize: '2vw',
            backgroundColor: 'white',
        } },
        React.createElement("div", { style: {
                textAlign: 'center',
                display: 'flex',
                flexDirection: 'column',
                alignItems: 'center',
            } },
            React.createElement("h1", null, "Please authorize this device"),
            code && (React.createElement("h1", { style: { display: 'flex', flexDirection: 'row' } },
                "Device code: ",
                !loading ? code : '',
                loading && (React.createElement("div", { style: { display: 'flex', flexDirection: 'row' } },
                    "\u00A0\u00A0\u00A0\u00A0",
                    React.createElement(Loader, null),
                    "\u00A0\u00A0\u00A0\u00A0")))),
            code && React.createElement("h4", null,
                "Waiting for authorization ",
                seconds,
                "s"),
            qrCode && !loading && (React.createElement("button", { onClick: () => window.open(qrCode, '_blank', 'popup,width=450,height=500'), style: {
                    backgroundColor: 'unset',
                    border: 'unset',
                    cursor: 'pointer',
                } },
                React.createElement(QRCodeSVG, { value: qrCode, size: size, level: "H" }))),
            loading && React.createElement(QRLoader, { size: size }),
            error && React.createElement("h1", { style: { color: 'red' } },
                "Error: ",
                error))));
}
function isSuccess(data) {
    return !(data === null || data === void 0 ? void 0 : data.error) && !!(data === null || data === void 0 ? void 0 : data.refresh_token);
}
function isPendingAuth(data) {
    return (data === null || data === void 0 ? void 0 : data.error) === 'authorization_pending';
}
function isSlowDown(data) {
    return (data === null || data === void 0 ? void 0 : data.error) === 'slow_down';
}
function deviceLoginReducer(state, action) {
    const { type, payload } = action;
    switch (type) {
        case 'enable':
            return Object.assign(Object.assign({}, state), { isDeviceCode: true });
        case 'loading':
            return Object.assign(Object.assign({}, state), { isDeviceCode: true, loading: true });
        case 'setValues':
            return Object.assign(Object.assign({}, state), { data: payload, error: undefined, loading: false });
        case 'error':
            return Object.assign(Object.assign({}, state), { error: payload });
        case 'disable':
            return {
                tenant: state.tenant,
                isDeviceCode: false,
                baseUrl: state.baseUrl,
                user: state.user,
            };
        default:
            return state;
    }
}
function hasValidRoles({ acceptedRolesArray, roles, }) {
    if (!acceptedRolesArray || acceptedRolesArray.length === 0) {
        return true;
    }
    const authorities = roles.map((r) => r.role);
    const rolesIntersection = acceptedRolesArray.filter((value) => authorities.includes(value));
    const isAuthorized = rolesIntersection.length > 0; //authorities.includes(role);
    return isAuthorized;
}
function checkDisallowedRoles({ roles, disallowedRoles, }) {
    if (!disallowedRoles || disallowedRoles.length === 0) {
        return [];
    }
    return roles
        .map((r) => r.role)
        .filter((role) => disallowedRoles.some((regExp) => regExp.test(role)));
}
export default function AuthWrapper(props) {
    const refreshStorageHandler = props.refreshStorageHandler || _refreshStorageHandler;
    const _configContext = useContext(ConfigContext);
    if (!_configContext) {
        return React.createElement(Fragment, null, "Configuration should be provided");
    }
    const config = _configContext;
    const { baseUrl, tenant } = config;
    const api = useMemo(() => axios.create({ baseURL: baseUrl }), [baseUrl]);
    const [auth, setAuth] = useState({
        loggedIn: false,
        UUID: '',
        authProvider: () => Promise.resolve({ token: '', UUID: '' }),
        signOut: onSignOut,
    });
    const [resolved, setResolved] = useState(false);
    const [state, dispatch] = useReducer(deviceLoginReducer, {
        isDeviceCode: false,
        tenant: tenant,
        baseUrl: baseUrl,
    });
    const [disallowedRoles, setDisallowedRoles] = useState(null);
    function onSignOut() {
        console.log('--- log out ----');
        setAuth({
            loggedIn: false,
            UUID: '',
            authProvider: () => Promise.resolve({ token: '', UUID: '' }),
            signOut: onSignOut,
            module: undefined,
        });
        setResolved(true);
        orderingCore.clearAuthData(config.tenant, refreshStorageHandler);
    }
    function getUserData(baseUrl, authProvider, setAuth) {
        return __awaiter(this, void 0, void 0, function* () {
            var _a;
            let haveToken = false;
            try {
                const { UUID, token } = yield authProvider();
                if (token) {
                    const userData = yield orderingCore.getLoggedUserData(baseUrl, token);
                    if (!hasValidRoles({
                        acceptedRolesArray: props.roles,
                        roles: userData.roles,
                    })) {
                        console.warn('--- USER HAS NO REQUIRED ROLES ----- ');
                        onSignOut();
                        setResolved(true);
                        alert('User has no required roles');
                        return false;
                    }
                    const userDisallowedRoles = checkDisallowedRoles({
                        roles: userData.roles,
                        disallowedRoles: props.disallowedRoles,
                    });
                    if (userDisallowedRoles.length) {
                        console.warn('--- USER HAS DISALLOWED ROLES ----- ', userDisallowedRoles);
                        (_a = props.appInsights) === null || _a === void 0 ? void 0 : _a.trackTrace({
                            message: 'Front user has disallowed roles',
                            severityLevel: 2, // Warning
                            properties: {
                                uuid: UUID,
                                roles: userData.roles,
                                userDisallowedRoles,
                            },
                        });
                        if (props.logoutOnDisallowedRoles) {
                            setDisallowedRoles(userDisallowedRoles);
                            onSignOut();
                            setResolved(true);
                            return false;
                        }
                    }
                    const module = yield getModuleConfig(config.baseUrl, token);
                    setAuth({
                        loggedIn: true,
                        email: userData.login,
                        UUID: UUID,
                        authProvider: authProvider,
                        signOut: onSignOut,
                        module,
                    });
                    haveToken = true;
                }
            }
            catch (err) {
                onSignOut();
            }
            setResolved(true);
            return haveToken;
        });
    }
    function deviceCodeLogin(moduleId, abortController) {
        return __awaiter(this, void 0, void 0, function* () {
            dispatch({ type: 'enable' });
            while (!abortController.signal.aborted) {
                try {
                    dispatch({ type: 'loading' });
                    // Get code
                    const { data: resData } = yield api.post('/auth-oauth2/oauth/device', { module: moduleId }, {
                        headers: {
                            Accept: 'application/json',
                            'X-Tenant': tenant,
                            'Content-Type': 'application/x-www-form-urlencoded',
                            Authorization: `Basic ${config.basicAuth}`,
                        },
                    });
                    const exp = Date.now() + resData.expires_in * 1000;
                    dispatch({ type: 'setValues', payload: Object.assign(Object.assign({}, resData), { exp }) });
                    let success = false;
                    while (!abortController.signal.aborted && Date.now() < exp - 2000) {
                        const { data: tokenData } = yield api.post('/auth-oauth2/oauth/token', {
                            device_code: resData.device_code,
                            grant_type: 'urn:ietf:params:oauth:grant-type:device_code',
                        }, {
                            headers: {
                                Accept: 'application/json',
                                'X-Tenant': tenant,
                                'Content-Type': 'application/x-www-form-urlencoded',
                                Authorization: `Basic ${config.basicAuth}`,
                            },
                            validateStatus: (status) => status < 500,
                        });
                        if (!isSuccess(tokenData)) {
                            yield new Promise((resolve) => setTimeout(resolve, resData.interval * 1000));
                        }
                        else {
                            yield refreshStorageHandler.setRefreshToken(tenant, tokenData.refresh_token);
                            orderingCore.setAuthData(tenant, tokenData, refreshStorageHandler);
                            success = true;
                            break;
                        }
                    }
                    if (success) {
                        break;
                    }
                }
                catch (e) {
                    const message = getErrorMessage(e);
                    if (message) {
                        dispatch({ type: 'error', payload: message });
                    }
                    yield new Promise((resolve) => setTimeout(resolve, 10000));
                    console.warn(e);
                    // continue loop
                }
            }
            dispatch({ type: 'disable' });
        });
    }
    useEffect(() => {
        if (auth.loggedIn) {
            return;
        }
        const abortController = new AbortController();
        function init() {
            return __awaiter(this, void 0, void 0, function* () {
                //console.log("AutWrapper init --- ");
                const authProvider = orderingCore.createAuthDataProvider({
                    BASE_URL: config.baseUrl,
                    TENANT: config.tenant,
                    BASIC_AUTH: config.basicAuth,
                    anonymousAuth: config.anonymousOnInit,
                }, refreshStorageHandler, false, (error) => {
                    setAuth((prev) => (Object.assign(Object.assign({}, prev), { loggedIn: false, UUID: '' })));
                    setResolved(false);
                });
                const params = new URLSearchParams(window.location.search);
                const moduleId = params.get('module') || config.moduleId;
                const { token } = yield authProvider();
                if (moduleId && !token) {
                    yield deviceCodeLogin(moduleId, abortController);
                }
                // Check for refreshToken in search params
                try {
                    const refreshToken = params.get('refreshToken') || config.refreshToken;
                    if (refreshToken) {
                        refreshStorageHandler.setRefreshToken(config.tenant, refreshToken);
                        // Remove refresh token from search params
                        params.delete('refreshToken');
                        const url = `${window.location.origin}${window.location.pathname}?${params.toString()}`;
                        window.history.replaceState(null, '', url);
                    }
                }
                catch (e) {
                    console.warn(e);
                }
                void getUserData(config.baseUrl, authProvider, setAuth);
                window.addEventListener('message', function (event) {
                    if (event.origin !== config.baseUrl)
                        return;
                    const authData = event.data;
                    console.log(authData);
                    if (authData.UUID === '') {
                        setAuth({
                            loggedIn: false,
                            UUID: '',
                            authProvider: authProvider,
                            signOut: onSignOut,
                        });
                    }
                    else {
                        orderingCore.setAuthData(config.tenant, {
                            UUID: authData.UUID,
                            access_token: authData.access_token,
                            refresh_token: authData.refresh_token,
                            expires_in: authData.expires_in,
                        }, refreshStorageHandler);
                        setAuth({
                            loggedIn: true,
                            UUID: authData.UUID,
                            authProvider: authProvider,
                            signOut: onSignOut,
                        });
                        getUserData(config.baseUrl, authProvider, setAuth);
                    }
                });
            });
        }
        void init();
        return () => abortController.abort();
    }, [auth === null || auth === void 0 ? void 0 : auth.loggedIn]);
    if (state.isDeviceCode) {
        const DeviceCodeComp = props.DeviceCodeComp || DefaultDeviceCodeComp;
        return React.createElement(DeviceCodeComp, Object.assign({}, state));
    }
    if (disallowedRoles === null || disallowedRoles === void 0 ? void 0 : disallowedRoles.length) {
        const DisallowedRolesComp = props.DisallowedRolesComp || DefaultDisallowedRolesComp;
        return React.createElement(DisallowedRolesComp, null);
    }
    return (React.createElement(Fragment, null,
        React.createElement(AuthContext.Provider, { value: Object.assign(Object.assign({}, auth), { resolved }) }, props.children)));
}
export function ShowWhenAuthenticated(props) {
    const { loggedIn } = useContext(AuthContext);
    return loggedIn ? React.createElement(Fragment, null, props.children) : null;
}
export function ShowWhenNotAuthenticated(props) {
    const { loggedIn } = useContext(AuthContext);
    return !loggedIn ? React.createElement(Fragment, null, props.children) : null;
}
export function ShowWhenNotAuthenticatedAnResolved(props) {
    const { loggedIn, resolved } = useContext(AuthContext);
    return !loggedIn && resolved ? React.createElement(Fragment, null, props.children) : null;
}
export function ShowWhenAuthenticating(props) {
    const { resolved } = useContext(AuthContext);
    return !resolved ? React.createElement(Fragment, null, props.children) : null;
}
function DefaultDisallowedRolesComp() {
    return (React.createElement("div", { style: {
            width: '100%',
            height: '100vh',
            display: 'flex',
            flexDirection: 'column',
            alignItems: 'center',
            justifyContent: 'center',
            fontSize: '2vw',
            backgroundColor: 'white',
        } },
        React.createElement("div", { style: {
                textAlign: 'center',
                display: 'flex',
                flexDirection: 'column',
                alignItems: 'center',
            } },
            React.createElement("h1", null, "Auth error 101. Contact support"))));
}
