var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import axios from 'axios';
import { getErrorMessage } from '../utils';
const tenant = '8724ef16-20c8-4008-b183-3504cedc38af';
const api = axios.create({ baseURL: 'https://ordering.3e.pl' });
const VALID_MODULE = 'kds-test'; // module with user
describe('Code Error Handler', () => {
    it('should return error if POST /code returns error in data', () => __awaiter(void 0, void 0, void 0, function* () {
        try {
            yield api.post('https://ordering.3e.pl/auth-oauth2/device/code', {
                code: 'ifXYPOxNRV',
                secret: '1N2Jyezg7uiky1SrksvISsYO',
                tenant,
            });
        }
        catch (e) {
            const message = getErrorMessage(e);
            expect(typeof message).toBe('string');
            expect(message).toBe('invalid_code');
        }
    }));
    it('should return message if POST /code returns message in data', () => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b;
        try {
            yield api.post('https://ordering.3e.pl/auth-oauth2/device/code', {
                code: 'invalid-code',
                secret: 'secret',
                tenant,
            });
        }
        catch (e) {
            const message = getErrorMessage(e);
            expect(typeof message).toBe('string');
            expect(message).toBe((_b = (_a = e.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.message);
        }
    }));
    it('should return undefined upon 408 (timeout) from POST /code', () => __awaiter(void 0, void 0, void 0, function* () {
        try {
            const { data: { code, secret }, } = yield api.get('/auth-oauth2/device/code', {
                params: {
                    tenant,
                    module: VALID_MODULE,
                },
            });
            yield api.post('/auth-oauth2/device/code', {
                code: code,
                secret: secret,
                tenant,
            });
        }
        catch (e) {
            const message = getErrorMessage(e);
            expect(message).toBe(undefined);
        }
    }), 120000);
    it('should return message if GET /code returns error in data', () => __awaiter(void 0, void 0, void 0, function* () {
        try {
            yield api.get('/auth-oauth2/device/code', {
                params: {
                    tenant: 'invalid-tenant',
                    module: 'invalid-module',
                },
            });
        }
        catch (e) {
            const message = getErrorMessage(e);
            expect(message).not.toBe(undefined);
        }
    }));
});
